
/******************************************************************************

Coral Tree nl/support.cc
Copyright (C) 1998 Infinite Monkeys Incorporated

This program is free software; you can redistribute it and/or modify
it under the terms of the Artistic License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
Artistic License for more details.

You should have received a copy of the Artistic License along with
this program (see meta/ART_LICE); if not, write to coral@imonk.com.

******************************************************************************/

#include "nl.h"

OSD_Output nlGlobalOutput;
unsigned long nlGlobalOutputInitialFlags = OSD_OUT_STDERR;

long NL_GLOBAL_STARTTIME = 0;
long NL_GLOBAL_LASTTIME = 0;

/******************************************************************************
void	nlPrint(long source, long type, char *message, char *file, long line)

NOTE:  This function has been reimplmented as a mapping to GPL_LOGGER.  Ignore
       the following description.

	print given message, file name, and line number

	this function can be adjusted to react to messages of
		specific sources and types
		ie. NL_APP,NT_WARN could be handled differently than
		NL_NL,NL_ERROR

	if type is not NL_LOG, use perror() instead of fprintf()

Keywords: debug
******************************************************************************/
/*PROTOTYPE*/
#if OSD_COMPILER==OSD_KR
void nlPrint(source, type, message, file, line)
	long source;
	long type;
	char *message;
	char *file;
	long line;
#else
void nlPrint(long source, long type, char *message, char *file, long line)
#endif
{
	source = source;
	static char tname[4][20]= 	
		{ "NL LOG", "NL WARNING", "NL ERROR", "NL ERROR" };
	GPL_LOGGER.Log(file, line, tname[type], message);
}
#if 0
{
	static initialized = FALSE;
	static char sourcename[3][20]= 	{ "Application", "Net Layer", "OS" 	};
	static char tname[4][20]= 	{ "Log", "Warning", "Error", "Fatal Error" 	};

	long errornum;
	errornum = OSD_ERRNO;
	char title[256];
	char *string;
	char *dummy;

	source = source;
	file = file;
	line = line;
	dummy=sourcename[0];dummy=tname[0];dummy=title;

	string=(char *)osdMalloc(strlen(message)+256);

	if (!initialized)
	{
		osdOutputOpen(&nlGlobalOutput);
		osdOutputSetFlags(&nlGlobalOutput, nlGlobalOutputInitialFlags);
	}

	if (NL_LOG != type)
	{
#if OSD_VERBOSITY != OSD_SILENT
		sprintf(string, "%s:%d \"%s\" ERRNUM: %d", file, line, message, errornum);
		sprintf(title, "NL %s %s", sourcename[source], tname[type]);
		osdOutputPrintLine(&nlGlobalOutput, title, string);
#endif

		if (NL_FATAL == type)
		{
			exit(1);
		}
	}
	else
	{
#ifdef VERBOSE_DEBUG
		fprintf(stderr, "Net Layer %s:%d %s \"%s\"\n", file, line, tname[type], message);
#endif
	}

	osdFree(string);
}
#endif



/******************************************************************************
void	nlInetAddrToString(char *string, int length, NL_HOST_INET_ADDR address)

	taken from abstract.dir/utility.cc

keywords: c, network, ip, abst_dep
******************************************************************************/
/*PROTOTYPE*/
#if OSD_COMPILER==OSD_KR
void nlInetAddrToString(string, length, address)
	char *string;
	int length;
	NL_HOST_INET_ADDR address;
#else
void nlInetAddrToString(char *string, int length, NL_HOST_INET_ADDR address)
#endif
{
	if (16 > length)
	{
		EXIT("string not long enough for inet address", -1);
	}

	sprintf(string, "%u.%u.%u.%u",

#if OSD_ENDIAN==OSD_LITTLE
		(address & 0x000000ff) >> 0,
		(address & 0x0000ff00) >> 8,
		(address & 0x00ff0000) >> 16,
		(address & 0xff000000) >> 24);
#else
		(address & 0xff000000) >> 24,
		(address & 0x00ff0000) >> 16,
		(address & 0x0000ff00) >> 8,
		(address & 0x000000ff) >> 0);
#endif

	return;
}



/******************************************************************************
NL_HOST_INET_ADDR	nlStringToInetAddr(char *string)

	taken from abstract.dir/utility.cc

keywords: c, network, ip, abst_dep
******************************************************************************/
/*PROTOTYPE*/
#if OSD_COMPILER==OSD_KR
NL_HOST_INET_ADDR nlStringToInetAddr(string)
	const char *string;
#else
NL_HOST_INET_ADDR nlStringToInetAddr(const char *string)
#endif
{
	NL_HOST_INET_ADDR address;
	uint32 b1, b2, b3, b4;
	int n;

	address = 0;
	n = sscanf(string, "%u.%u.%u.%u", &b1, &b2, &b3, &b4);
#if OSD_ENDIAN==OSD_LITTLE
	address = address | (b4 << 24);
	address = address | (b3 << 16);
	address = address | (b2 << 8);
	address = address | (b1 << 0);
#else
	address = address | (b1 << 24);
	address = address | (b2 << 16);
	address = address | (b3 << 8);
	address = address | (b4 << 0);
#endif

	if (4 != n)
	{
		return (NL_HOST_INET_ADDR)NULL;
	}

	return address;
	}

/******************************************************************************
void nlSIGPIPE_Handler(...)

******************************************************************************/
OSD_SignalHandler(nlSIGPIPE_Handler)
{
	/* DO NOTHING, JUST CATCH TO PREVENT EXIT */
	NL_PRINT(NL_OS, NL_ERROR, "caught broken pipe");
	return;
}

/******************************************************************************
uint32	nlIpSocketStartup(void)

******************************************************************************/
/*PROTOTYPE*/
#if OSD_COMPILER==OSD_KR
uint32 nlIpSocketStartup(void)
#else
uint32 nlIpSocketStartup(void)
#endif
{
#if OSD_OSTYPE==OSD_WIN32
	WORD wVersionRequested;
	WSADATA wsaData;
	int err;

	wVersionRequested = MAKEWORD(NL_HIGH_BYTE, NL_LOW_BYTE);
	
	err = WSAStartup(wVersionRequested, &wsaData);

	if (0 != err)
	{
		fprintf(stderr, "Could not find usable winsock.dll\n");
		EXIT("NT socket startup failed", -1);
	}

	if (NL_LOW_BYTE != LOBYTE(wsaData.wVersion) ||
		NL_HIGH_BYTE != HIBYTE(wsaData.wVersion))
	{
		fprintf(stderr, "Could not find usable winsock.dll\n");
		WSACleanup();
		EXIT("NT socket startup failed", -1);
	}

	return 1;
#else
	static long first = 1;

	if (first)
	{
		first = 0;
		OSD_SetSignalHandler(nlSIGPIPE_Handler, SIGPIPE);
	}

	return 1;
#endif
}

/******************************************************************************
uint32	nlIpSocketCleanup(void)

******************************************************************************/
/*PROTOTYPE*/
#if OSD_OSTYPE==OSD_WIN32
#if OSD_COMPILER==OSD_KR
uint32 nlIpSocketCleanup(void)
#else
uint32 nlIpSocketCleanup(void)
#endif
{
	int err;

	err = WSACleanup();

	ERR_IF(err, "Could not clean up sockets");

	return 1;
}
#endif


