/******************************************************************************

Coral Tree dna/bus.cc
Copyright (C) 1998 Infinite Monkeys Incorporated

This program is free software; you can redistribute it and/or modify
it under the terms of the Artistic License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
Artistic License for more details.

You should have received a copy of the Artistic License along with
this program (see meta/ART_LICE); if not, write to coral@imonk.com.

******************************************************************************/

#include <gpl/gpl.h>
#include <wba/wba.h>
#include <dna/dna.h>

/******************************************************************************
class DNA_Bus
******************************************************************************/

/***********************************************************************
GPL_Boolean DNA_Bus::IsValid(void) const
***********************************************************************/
GPL_Boolean DNA_Bus::IsValid(void) const
{
	register DNA_BusRep *r = (DNA_BusRep *) rep;
	if ((DNA_BusRep *) GPL_BAD_ADDRESS != r)
	{
		if (DNA_BUS_VALID == (r->state & DNA_BUS_VALID))
		{
			return TRUE;
		}
	}
	return FALSE;
}

/***********************************************************************
DNA_BusImp *DNA_Bus::GetImp(void) const
***********************************************************************/
DNA_BusImp *DNA_Bus::GetImp(void) const
{
	return GPL_Reference::IsValid() ? ((DNA_BusRep *) rep)->imp : (DNA_BusImp *) GPL_BAD_ADDRESS;
}

/***********************************************************************
long DNA_Bus::Compare(const DNA_Bus & bus) const
***********************************************************************/
long DNA_Bus::Compare(const DNA_Bus & bus) const
{
	register const DNA_BusImp *i = GetImp();
	register const DNA_BusImp *j = bus.GetImp();
	if ((DNA_BusImp *) GPL_BAD_ADDRESS == i ||
		(DNA_BusImp *) GPL_BAD_ADDRESS == j)
	{
		return i - j;
	}
	return i->Compare(j);
}

/***********************************************************************
void DNA_Bus::SetImp(DNA_BusImp *i)
***********************************************************************/
void DNA_Bus::SetImp(DNA_BusImp *i)
{
	GPL_ENSURE(DNA_BusRep, r);
	r->imp = i;
}

/***********************************************************************
void DNA_Bus::Reset(void)
***********************************************************************/
void DNA_Bus::Reset(void)
{
	register DNA_BusRep *r = (DNA_BusRep *) rep;
	if ((DNA_BusRep *) GPL_BAD_ADDRESS != r)
	{
		r->state = DNA_BUS_VALID;
		if ((DNA_BusImp *) GPL_BAD_ADDRESS != r->imp)
		{
			r->imp->Reset();
		}
	}
}

/***********************************************************************
GPL_Boolean DNA_Bus::Read(void *buffer, const unsigned long size,
							unsigned long & length)
***********************************************************************/
GPL_Boolean DNA_Bus::Read(void *buffer, const unsigned long size,
							unsigned long & length)
{
	GPL_ENSURE(DNA_BusRep, r);
	if (!DNA_CHECK_STATE(DNA_BUS_VALID))
	{
		error.Set(DNA_ERROR_BAD_STATE);
		DNA_LOG_ERROR;
		return FALSE;
	}

	if (DNA_NO_IMP)
	{
		error.Set(DNA_ERROR_BAD_IMPLEMENTATION);
		DNA_LOG_ERROR;
		return FALSE;
	}

	if (!r->imp->Read(buffer, size, length, error))
	{
		switch(error.GetType())
		{
		case DNA_ERROR_NONE:
			// Falls through...
		case DNA_ERROR_INSUFFICIENT_MEMORY:
			// Falls through...
		case DNA_ERROR_NO_DATA_AVAILABLE:
			break;
		case DNA_ERROR_BAD_HANDLE:
			DNA_CLEAR_STATE(DNA_BUS_VALID);
			break;
			break;
		case DNA_ERROR_BAD_MESSAGE:
			// Falls through...
		case DNA_ERROR_BAD_ADDRESS:
			// Falls through...
		case DNA_ERROR_INTERRUPTED_CALL:
			// Falls through...
		case DNA_ERROR_IO_ERROR:
			break;
		case DNA_ERROR_BAD_IMPLEMENTATION:
			// Falls through...
		case DNA_ERROR_BAD_STATE:
			// Falls through...
		default:
			DNA_CLEAR_STATE(DNA_BUS_VALID);
			DNA_LOG_ERROR;
			break;
		}
		return FALSE;
	}

	return TRUE;
}

/***********************************************************************
GPL_Boolean DNA_Bus::Read(void *buffer, const unsigned long size,
					const long timeout, unsigned long & length)
***********************************************************************/
GPL_Boolean DNA_Bus::Read(void *buffer, const unsigned long size,
					const long timeout, unsigned long & length)
{
	GPL_ENSURE(DNA_BusRep, r);
	if (!DNA_CHECK_STATE(DNA_BUS_VALID))
	{
		error.Set(DNA_ERROR_BAD_STATE);
		DNA_LOG_ERROR;
		return FALSE;
	}

	if (DNA_NO_IMP)
	{
		error.Set(DNA_ERROR_BAD_IMPLEMENTATION);
		DNA_LOG_ERROR;
		return FALSE;
	}

	if (!r->imp->Read(buffer, size, timeout, length, error))
	{
		switch(error.GetType())
		{
		case DNA_ERROR_NONE:
			// Falls through...
		case DNA_ERROR_INSUFFICIENT_MEMORY:
			// Falls through...
		case DNA_ERROR_NO_DATA_AVAILABLE:
			break;
		case DNA_ERROR_BAD_HANDLE:
			DNA_CLEAR_STATE(DNA_BUS_VALID);
			break;
		case DNA_ERROR_BAD_MESSAGE:
			// Falls through...
		case DNA_ERROR_BAD_ADDRESS:
			// Falls through...
		case DNA_ERROR_INTERRUPTED_CALL:
			// Falls through...
		case DNA_ERROR_IO_ERROR:
			break;
		case DNA_ERROR_BAD_IMPLEMENTATION:
			// Falls through...
		case DNA_ERROR_BAD_STATE:
			// Falls through...
		default:
			DNA_CLEAR_STATE(DNA_BUS_VALID);
			DNA_LOG_ERROR;
			break;
		}
		return FALSE;
	}

	return TRUE;
}

/***********************************************************************
GPL_Boolean DNA_Bus::Write(void *buffer, const unsigned long size,
							unsigned long & length)
***********************************************************************/
GPL_Boolean DNA_Bus::Write(void *buffer, const unsigned long size,
							unsigned long & length)
{
	GPL_ENSURE(DNA_BusRep, r);
	if (!DNA_CHECK_STATE(DNA_BUS_VALID))
	{
		error.Set(DNA_ERROR_BAD_STATE);
		DNA_LOG_ERROR;
		return FALSE;
	}

	if (DNA_NO_IMP)
	{
		error.Set(DNA_ERROR_BAD_IMPLEMENTATION);
		DNA_LOG_ERROR;
		return FALSE;
	}

	if (!r->imp->Write(buffer, size, length, error))
	{
		switch(error.GetType())
		{
		case DNA_ERROR_NONE:
			// Falls through...
		case DNA_ERROR_INSUFFICIENT_MEMORY:
			// Falls through...
		case DNA_ERROR_NO_DATA_AVAILABLE:
			break;
		case DNA_ERROR_BAD_HANDLE:
			DNA_CLEAR_STATE(DNA_BUS_VALID);
			DNA_LOG_ERROR;
			break;
		case DNA_ERROR_BAD_MESSAGE:
			// Falls through...
		case DNA_ERROR_BAD_ADDRESS:
			// Falls through...
		case DNA_ERROR_INTERRUPTED_CALL:
			// Falls through...
		case DNA_ERROR_IO_ERROR:
			break;
		case DNA_ERROR_BAD_IMPLEMENTATION:
			// Falls through...
		case DNA_ERROR_BAD_STATE:
			// Falls through...
		default:
			DNA_CLEAR_STATE(DNA_BUS_VALID);
			DNA_LOG_ERROR;
			break;
		}
		return FALSE;
	}

	return TRUE;
}

/***********************************************************************
GPL_Boolean DNA_Bus::Print(GPL_String & string,
							const GPL_Verbosity verbosity) const
***********************************************************************/
GPL_Boolean DNA_Bus::Print(GPL_String & string,
							const GPL_Verbosity verbosity) const
{
	if (GPL_Reference::IsValid())
	{
		register DNA_BusRep *r = (DNA_BusRep *) rep;
		GPL_String s;
		if ((DNA_BusImp *) GPL_BAD_ADDRESS == r->imp)
		{
			string.Append("null");
		}
		else
		{
			s.Printf("%s(%p)", r->imp->GetClassName(), (void *) r->imp);
			string.Append(s);
		}
		string.Append(',');
		if (r->state & DNA_BUS_VALID)
		{
			string.Append("VALID");
		}
	}
	return TRUE;
}

/***********************************************************************
GPL_Boolean DNA_Bus::Parse(const GPL_String & string)
***********************************************************************/
GPL_Boolean DNA_Bus::Parse(const GPL_String & string)
{
	// DNA objects may not be parsed.
	return FALSE;
}

GPL_OBJECT_IMPLEMENTATION(DNA_Bus, 0x40070004, "DNA_Bus");

