#ifndef DNA_BUS_H
#define DNA_BUS_H

/******************************************************************************

Coral Tree dna/bus.h
Copyright (C) 1998 Infinite Monkeys Incorporated

This program is free software; you can redistribute it and/or modify
it under the terms of the Artistic License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
Artistic License for more details.

You should have received a copy of the Artistic License along with
this program (see meta/ART_LICE); if not, write to coral@imonk.com.

******************************************************************************/

//#include <gpl/gpl.h>
#include <dna/error.h>

#define	DNA_BLOCK_INDEFINITELY	-1

/***********************************************************************
class DNA_BusImp

Network-layer data bus implementation.
***********************************************************************/
class DNA_BusImp : public GPL_Object
{
	public:
		DNA_BusImp(void) {}
		virtual ~DNA_BusImp(void) {}

		virtual long Compare(const DNA_BusImp *imp) const
		{
			void *i = (void *)(imp);i = i;
			return 0;
		}

		virtual void Reset(void) {}
		virtual GPL_Boolean Read(void *buffer, const unsigned long size,
							unsigned long & length, DNA_Error & error)
		{
			buffer = buffer;
			unsigned long s = size; s = s;
			length = length;
			error = error;
			return FALSE;
		}
		virtual GPL_Boolean Read(void *buffer, const unsigned long size,
							const long timeout, unsigned long & length,
							DNA_Error & error)
		{
			buffer = buffer;
			unsigned long s = size; s = s;
			long to = timeout; to = to;
			length = length;
			error = error;
			return FALSE;
		}
		virtual GPL_Boolean Write(void *buffer, const unsigned long size,
							unsigned long & length, DNA_Error & error)
		{
			buffer = buffer;
			unsigned long s = size; s = s;
			length = length;
			error = error;
			return FALSE;
		}

		// GPL_Object
		GPL_OBJECT_ABSTRACT_INTERFACE(DNA_BusImp);
};

/***********************************************************************
enum DNA_BusState
***********************************************************************/
enum DNA_BusState
{
	DNA_BUS_VALID	=	0x00000001
};

#define	DNA_LOG_ERROR	wbaLog("DNA LOG", error.ToString().GetData())

/***********************************************************************
class DNA_Bus

Network-layer data bus.
***********************************************************************/
class DNA_Bus : public GPL_Object, public GPL_Reference, public GPL_Depictable
{
	public:
		DNA_Bus(void) {}
		DNA_Bus(const DNA_Bus & b) { Attach(b); }
		virtual ~DNA_Bus(void) { Detach(); }

		DNA_Bus & operator=(const DNA_Bus & b) { Detach(); Attach(b); return *this; }
		GPL_Boolean operator==(const DNA_Bus & b) const { return 0 == Compare(b); }

		virtual GPL_Boolean IsValid(void) const;
		virtual const DNA_Error & GetError(void) const { return error; }
		virtual DNA_BusImp *GetImp(void) const;

		virtual long Compare(const DNA_Bus & bus) const;

		virtual void SetImp(DNA_BusImp *i);

		virtual void Reset(void);
		virtual GPL_Boolean Read(void *buffer, const unsigned long size,
							unsigned long & length);
		virtual GPL_Boolean Read(void *buffer, const unsigned long size,
							const long timeout, unsigned long & length);
		virtual GPL_Boolean Write(void *buffer, const unsigned long size,
							unsigned long & length);

		// GPL_Depictable
		virtual GPL_Boolean Print(GPL_String & string,
								const GPL_Verbosity verbosity) const;
		virtual GPL_Boolean Parse(const GPL_String & string);

		// GPL_Object
		GPL_OBJECT_INTERFACE(DNA_Bus);

	protected:
		// This is the error state for the object.  It contains
		// per-object errors that may be set by the object, by
		// its implementation, or by objects encapsulated by
		// the implementation.  Keeping the error local to the
		// object yet still available to a variety of associated
		// objects poses some difficulties.  Using its address
		// is fraught with peril, however, since member-wise
		// copies bring with them the address of the now invalid
		// data member in the enclosing object.  GPL_Pointer
		// seemed to be too heavy for this application.  Instead,
		// error is passed as a non-const reference all over the
		// place.  While causing argument bloat, it should be
		// as fast as raw pointers and will always be correct.
		DNA_Error error;

		virtual void Privatize(void) {}
};

/******************************************************************************
class DNA_BusRep

Network-layer data bus representation.
******************************************************************************/
class DNA_BusRep : public GPL_ReferenceRep
{
	public:
		unsigned long state;
		DNA_BusImp *imp;

		DNA_BusRep(void) : state(DNA_BUS_VALID),
							imp((DNA_BusImp *) GPL_BAD_ADDRESS) {}
		virtual ~DNA_BusRep(void)
		{
			if ((DNA_BusImp *) GPL_BAD_ADDRESS != imp)
			{
				delete imp;
			}
		}
};

#define DNA_NO_IMP	((DNA_BusImp *) GPL_BAD_ADDRESS == ((DNA_BusRep *) rep)->imp)
#define	DNA_SET_STATE(x)		((DNA_BusRep *) rep)->state = ((DNA_BusRep *) rep)->state | (x)
#define	DNA_CLEAR_STATE(x)		((DNA_BusRep *) rep)->state = ((DNA_BusRep *) rep)->state & ~(x)
#define	DNA_CHECK_STATE(x)	((x) == (((DNA_BusRep *) rep)->state & (x)))

#endif /* DNA_BUS_H */
