#ifndef DNA_CLIENT_H
#define DNA_CLIENT_H

/******************************************************************************

Coral Tree dna/client.h
Copyright (C) 1998 Infinite Monkeys Incorporated

This program is free software; you can redistribute it and/or modify
it under the terms of the Artistic License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
Artistic License for more details.

You should have received a copy of the Artistic License along with
this program (see meta/ART_LICE); if not, write to coral@imonk.com.

******************************************************************************/

#include <gpl/gpl.h>

#include <dna/connect.h>
#include <dna/ippoint.h>
#include <dna/udp.h>
#include <dna/message.h>
#include <dna/handler.h>

/******************************************************************************
class DNA_Client

Network connection client.
******************************************************************************/
class DNA_Client
{
	public:
		DNA_Client(void);
		virtual ~DNA_Client(void) {}

		virtual const DNA_Error & GetError(void) const
		{
			return connection.GetError();
		}
		long GetTimeout(void) const { return timeout; }
		virtual GPL_Boolean IsConnected(void)
		{
			return connection.IsConnected();
		}

		void SetTimeout(const long t) { timeout = t; }

		virtual GPL_Boolean Initialize(void);
		virtual void Finalize(void);
		virtual GPL_Boolean Reconnect(void);

		DNA_IPPoint *GetDestination(void) const
		{
			return (DNA_IPPoint *) connection.GetDestination();
		}
		void SetDestination(DNA_IPPoint *d) { connection.SetDestination(d); }

	protected:
		void SetImp(DNA_UDPBindingImp *i) { connection.SetImp(i); }

		virtual void Process(void);
		virtual void ProcessMessage(DNA_Message & message);
		virtual GPL_Boolean Request(DNA_Message & message);
		virtual GPL_Boolean Reply(DNA_Message & message);
		virtual	GPL_Boolean Receive(DNA_Message & message);

		virtual unsigned long GetNumberHandlers(void) const;
		virtual void AppendHandler(const GPL_Pointer<DNA_MessageHandler> & handler);
		virtual void InsertHandler(const GPL_Pointer<DNA_MessageHandler> & handler, const unsigned long at = 0);

	private:
		long timeout;
		DNA_Connection connection;
		GPL_List< GPL_Pointer<DNA_MessageHandler> > handlers;
};

#endif /* DNA_CLIENT_H */
