#ifndef DNA_SELECTOR_H
#define DNA_SELECTOR_H

/******************************************************************************

Coral Tree dna/ipmplex.h
Copyright (C) 1998 Infinite Monkeys Incorporated

This program is free software; you can redistribute it and/or modify
it under the terms of the Artistic License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
Artistic License for more details.

You should have received a copy of the Artistic License along with
this program (see meta/ART_LICE); if not, write to coral@imonk.com.

******************************************************************************/

#include <gpl/gpl.h>

#include <dna/binding.h>

/***********************************************************************
enum DNA_IPMultiplexableFlags
***********************************************************************/
enum DNA_IPMultiplexableFlags
{
	DNA_IPMULTIPLEXABLE_READ		= 1,
	DNA_IPMULTIPLEXABLE_WRITE		= 2,
	DNA_IPMULTIPLEXABLE_EXCEPTION	= 4
};

/***********************************************************************
class DNA_IPMultiplexable

IPMultiplexable network-layer binding.
***********************************************************************/
class DNA_IPMultiplexable
{
	public:
		SOCKET_HANDLE handle;
		unsigned long flags;

		DNA_IPMultiplexable(SOCKET_HANDLE h = INVALID_SOCKET_HANDLE,
							const unsigned long f = 0) : handle(h), flags(f) {}

		GPL_Boolean operator==(const DNA_IPMultiplexable & m) const
		{
			return handle == m.handle;
		}
};

#define	DNA_MULTIPLEXOR_BLOCK	-1

/******************************************************************************
class DNA_IPMultiplexor

Network-layer binding selector.
******************************************************************************/
class DNA_IPMultiplexor
{
	public:
		DNA_IPMultiplexor(void);
		virtual ~DNA_IPMultiplexor(void);

		long GetTimeout(void) const { return timeout; }

		void SetTimeout(const long t) { timeout = t; }

		unsigned long Wait(GPL_List<DNA_IPMultiplexable> & ready)
		{
			return Wait(timeout, ready);
		}
		unsigned long Wait(const long t, GPL_List<DNA_IPMultiplexable> & ready);

		void Add(SOCKET_HANDLE handle,
					const unsigned long flags = DNA_IPMULTIPLEXABLE_READ);
		void Remove(SOCKET_HANDLE handle);

	protected:
		long timeout;
		fd_set readfds;
		fd_set writefds;
		fd_set exceptfds;
		GPL_List<DNA_IPMultiplexable> multiplexables;
};

#endif /* DNA_SELECTOR_H */
