#ifndef DNA_IPSOCKET_H
#define DNA_IPSOCKET_H

/******************************************************************************

Coral Tree dna/ipsocket.h
Copyright (C) 1998 Infinite Monkeys Incorporated

This program is free software; you can redistribute it and/or modify
it under the terms of the Artistic License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
Artistic License for more details.

You should have received a copy of the Artistic License along with
this program (see meta/ART_LICE); if not, write to coral@imonk.com.

******************************************************************************/

#include <osd/osd.h>
#include <gpl/gpl.h>

#include <dna/binding.h>
#include <dna/ippoint.h>

/******************************************************************************
class DNA_IPSocket

IP socket.
******************************************************************************/
class DNA_IPSocket : public GPL_Object
{
	public:
		DNA_IPSocket(const int d = AF_INET, const int t = SOCK_STREAM);
		DNA_IPSocket(const DNA_IPSocket & s);
		virtual ~DNA_IPSocket(void);

		virtual SOCKET_HANDLE GetSocketHandle(void) const { return handle; }

		virtual void SetBackLog(const unsigned long b);
		GPL_Boolean SetOptions(const int level, const int option,
								const void *buffer, const int length);
		int Ioctl(long cmd, void *arg);

		virtual void Reset(void);
		virtual GPL_Boolean Accept(DNA_IPPoint *destination, DNA_IPSocket & s,
									DNA_Error & error);
		virtual GPL_Boolean Connect(DNA_IPPoint *destination, DNA_Error & error);
		virtual GPL_Boolean Disconnect(DNA_Error & error);
		virtual GPL_Boolean Open(DNA_Error & error);
		virtual GPL_Boolean Bind(DNA_Point *point, DNA_Error & error);
		virtual GPL_Boolean Close(DNA_Error & error);
		virtual GPL_Boolean ReadFrom(DNA_IPPoint *destination, void *buffer,
										const unsigned long size,
										unsigned long & length,
										DNA_Error & error);
		virtual GPL_Boolean ReadFrom(DNA_IPPoint *destination, void *buffer,
										const unsigned long size,
										const long timeout,
										unsigned long & length,
										DNA_Error & error);
		virtual GPL_Boolean Read(void *buffer, const unsigned long size,
									unsigned long & length, DNA_Error & error);
		virtual GPL_Boolean Read(void *buffer, const unsigned long size,
									const long timeout, unsigned long & length,
									DNA_Error & error);
		virtual GPL_Boolean WriteTo(DNA_IPPoint *destination, void *buffer,
										const unsigned long size,
										unsigned long & length,
										DNA_Error & error);
		virtual GPL_Boolean Write(void *buffer, const unsigned long size,
									unsigned long & length, DNA_Error & error);

		static void GlobalInitialize(void);
		static void GlobalFinalize(void);

		// GPL_Object
		GPL_OBJECT_INTERFACE(DNA_IPSocket);

	protected:
		int domain;
		int type;
		SOCKET_HANDLE handle;

	private:
		static GPL_Boolean Close(SOCKET_HANDLE & handle);
};

#endif /* DNA_IPSOCKET_H */
