#ifndef DNA_TSERVER_H
#define DNA_TSERVER_H

/******************************************************************************

COPYRIGHT_HEADER

******************************************************************************/

/******************************************************************************
DNA_ThreadedServer is a concurrent server implemented with threads.  
******************************************************************************/

class DNA_ThreadedServer;

class DNA_ProcessMessageTask : public MTE_Task
{
	private:
		DNA_ThreadedServer	*server;
		SOCKET_HANDLE		handle;
		MTE_Mutex			cond_var; 	// acts as sync for restarts
	public:
		DNA_ProcessMessageTask()
		{
			server = NULL;
			SetDetach(TRUE);
		}
virtual	~DNA_ProcessMessageTask(){}
		void Begin(DNA_ThreadedServer *s, SOCKET_HANDLE h)
		{
			server = s;
			handle = h;
			if(GetState() == MTE_TASK_DONE)
			{
				cond_var.Unlock();
			}
			else
			{
				Start();
			}
		}
virtual	void Work(void);
};

class DNA_ThreadedServer : public DNA_Server
{
	public:
		DNA_ThreadedServer(void);
virtual	~DNA_ThreadedServer(void) {}

virtual	GPL_Boolean Initialize(void);
virtual	void Finalize(void);

	public:
virtual	void Wait(void);
virtual	void TendConnection(GPL_Opaque handle);

		void AddFreeTask(DNA_ProcessMessageTask *task)
		{
			MTE_Guard guard(&mutex);
			taskPool.AppendNode(task);
		}
		DNA_ProcessMessageTask *GetFreeTask(void)
		{
			MTE_Guard guard(&mutex);
			DNA_ProcessMessageTask *task;
			taskPool.ToHead();
			task = taskPool.PostIncrement();
			if(task) 
			{ 
				taskPool.RemoveNode(task);
			}
			else
			{
				task = new DNA_ProcessMessageTask();
			}
			return task;
		}

	private:
		// No passing around of servers.
		DNA_ThreadedServer(const DNA_ThreadedServer & server);
		DNA_ThreadedServer & operator=(const DNA_ThreadedServer & server);

		GPL_DoubleList<DNA_ProcessMessageTask>	taskPool;
		MTE_Mutex								mutex;
};


#endif /* DNA_TSERVER_H */
